'From Haver 6.0 [latest update: #5202] on 29 May 2022 at 8:02:04 pm'!
'Description Convert the running Cuis image to an Haver image.'!
!provides: 'Haverize' 1 81!
SystemOrganization addCategory: 'Haverize'!


!classDefinition: #HaverizePackage category: 'Haverize'!
CodePackage subclass: #HaverizePackage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Haverize'!
!classDefinition: 'HaverizePackage class' category: 'Haverize'!
HaverizePackage class
	instanceVariableNames: 'isUnzippedDistribution'!


!HaverizePackage commentStamp: '<historical>' prior: 0!
I am a code package class that installs all the haver change sets and packages.!

!HaverizePackage class methodsFor: 'integration' stamp: 'KLG 4/10/2022 15:39:04'!
integrateEnvironmentPackages
	"Integrate all packages related to environments."

	^ self namesOfPackagesAndCategoriesToIntegrate pairsDo: [ :packageName :newSystemCategory |
		Modules>>#PackageIntegrators>>#PackageIntegrator ::
			forPackage: packageName  systemCategory: newSystemCategory ::
				integrate ]! !

!HaverizePackage class methodsFor: 'integration' stamp: 'KLG 2/19/2022 19:51:06'!
integrateExtensionPackages
	"Integrate the packages that a re pure extensions."

	^ self namesOfPackagesToIntegrate do: [ :packageName |
		Modules>>#PackageIntegrators>>#PackageIntegrator ::
			forPackage: packageName  ::
				integrate ]! !

!HaverizePackage class methodsFor: 'integration' stamp: 'KLG 4/20/2022 13:54:31'!
integrateHaverPackages
	"Integrate all the haver packages."

	self integrateExtensionPackages.
	self integrateEnvironmentPackages.
	self integrateModules! !

!HaverizePackage class methodsFor: 'integration' stamp: 'KLG 2/19/2022 19:59:14'!
integrateModules
	"Integrate all modules."

	^ self namesOfModulesAndCategoriesToIntegrate pairsDo: [ :moduleName : newSystemCategory |
		Modules>>#PackageIntegrators>>#ModuleIntegrator ::
			forModule: (Modules >> moduleName)  systemCategory: newSystemCategory ::
				integrate ]! !

!HaverizePackage class methodsFor: 'integration' stamp: 'KLG 5/4/2022 12:17:13'!
namesOfModulesAndCategoriesToIntegrate
	"Answer pairs of module names an catgory names to integrate."

	^ #(
		ClassBuilding 'Haver-Modules ClassBuilding'
		HaverCompiler 'Haver-Compiler'
		Modules 'Haver-Modules'
		ModulesRefactorings 'Haver-Refactorings'
		ModulesTools 'Haver-Tools'
		ModuleAwareTools 'Haver-Tools'
		LegacyButtons 'Haver-Widgets Legacy'
		Buttons 'Haver-Widgets'
		DNDIndentingListItemMorph 'Haver-Widgets'
		SystemMorphs 'Haver-Widgets'
		ActionBuilder 'Haver-Actions'
		PackageIntegrators 'Haver-Integration'
	) ! !

!HaverizePackage class methodsFor: 'integration' stamp: 'KLG 5/25/2022 21:52:10'!
namesOfPackagesAndCategoriesToIntegrate
	"Answer pairs of package names an catgory names to integrate."

	^ #(
		'EnvironmentAwareness' 'Haver-Environments Base'
		'Environments' 'Haver-Environments Deprecated'
		'EnvironmentsBase' 'Haver-Environments Base'
		'SimpleEnvironments' 'Haver-Environments Simple'
		'Haverize' 'Haver-Haverize'
		'MoreMouseButtons' 'Haver-More Mouse Buttons'
	) ! !

!HaverizePackage class methodsFor: 'integration' stamp: 'KLG 5/25/2022 21:51:10'!
namesOfPackagesToIntegrate
	"Answer the name sof packages that should be integrated."

	^ #('PreferencesMenu' 'Accessability' 'KeyboardFocus' 'MoreMouseActions')! !

!HaverizePackage class methodsFor: 'preferences' stamp: 'KLG 5/17/2022 12:10:00'!
ensureHaverPreferences
	"Ensur ethe proper preferences seetings for Haver."

	PreferenceNG
		at: #menusShowSubSectionTitles put: true;
		at: #tapAndHoldEmulatesButton2 put: false;
		at: #checkForSlips put: false;
		at: #stylingWithEmphasisInWorkspaces put: true;
		at: #biggerCursors put: true;
		at: #fullScreenLeavesDeskMargins put: false;
		at: #dismissAllOnOptionClose put: true! !

!HaverizePackage class methodsFor: 'distribution' stamp: 'KLG 9/23/2021 20:30:18'!
unzippedDistributionIndicatorFilename
	"This filename is used as an indiactor for an unzipped distribution."

	^ 'IsHaverDistribution.txt'! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 4/21/2022 11:47:37'!
computePackageDirectoryRoot
	"Compute the root of the package directory hierrachy."

	^ self isUnzippedDistribution
		ifFalse: [
			self installedPackages
				detect: [ :package | package isKindOf: self ]
				ifNone: [ ^ nil ] :: 
					packageDirectoryName asDirectoryEntry parent ]
		ifTrue: [ DirectoryEntry smalltalkImageDirectory parent ]! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 4/13/2022 21:32:44'!
hasIntegrateCommandlineOption
	"Answer true if the start up arguments contain an integrate option."

	^ Smalltalk startUpArguments includes: '--integrate'! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 4/21/2022 11:49:29'!
installHaver
	"Install all the haver packages."

	self computePackageDirectoryRoot
		ifNotNil: [ :rootDirectory |
			self isUnzippedDistribution
				ifFalse: [
					self installHaverPackagesFromRepository: rootDirectory ]
				ifTrue: [
					self installHaverPackagesFromUnzippedDistribution: rootDirectory ].
			self shouldIntegrate ifTrue: [		self integrateHaverPackages ] ]
		ifNil: [
			Transcript
				cr;
				show: 'Haver is already installed!!'.
			self halt: 'Double Install!!' ]! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 5/25/2022 21:54:01'!
installHaverPackagesFromRepository: aRootDirectory
	"Install all the haver packages from the Mercurial repository."

	| environmentsDirectory coreChangesDirectory klgPackagesDirectory haverDirectory |
	environmentsDirectory _ aRootDirectory / 'environments'.
	coreChangesDirectory _ environmentsDirectory / 'coreChanges'.
	haverDirectory _ aRootDirectory / 'haver'.
	"File in all the check files:"
	(coreChangesDirectory // 'CuisCoreEnvironmentSupport.cs.st.chk.st') readStream fileIn.
	(environmentsDirectory // 'EnvironmentAwareness.pck.st.chk.st') readStream fileIn.
	(environmentsDirectory // 'EnvironmentsBase.pck.st.chk.st') readStream fileIn.
	(environmentsDirectory // 'SimpleEnvironments.pck.st.chk.st') readStream fileIn.
	klgPackagesDirectory _ aRootDirectory / 'klg-packages'.
	self killAllBrowsers.
	(coreChangesDirectory // 'CuisCoreEnvironmentSupport.cs.st') readStream fileIn.
	Feature
		require: (klgPackagesDirectory // 'Accessability.pck.st') pathName;
		require: (klgPackagesDirectory // 'KeyboardFocus.pck.st') pathName;
		require: (environmentsDirectory // 'SimpleEnvironments.pck.st') pathName;
		require: (environmentsDirectory // 'Environments.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'Modules.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'ModulesRefactorings.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'HaverCompiler.pck.st') pathName;
		require: (haverDirectory / 'ui' // 'DNDIndentingListItemMorph.pck.st') pathName;
		require: (haverDirectory / 'actions' // 'ActionBuilder.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'ModulesTools.pck.st') pathName;
		"F:
		require: (haverDirectory / 'transactions' // 'WriteBarrier.pck.st') pathName;
		require: (haverDirectory / 'transactions' // 'Transactions.pck.st') pathName"
		require: (haverDirectory / 'modules' // 'ClassBuilding.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'ModuleAwareTools.pck.st') pathName;
		require: (klgPackagesDirectory // 'PreferencesMenu.pck.st') pathName;
		require: (klgPackagesDirectory // 'MoreMouseButtons.pck.st') pathName	;
		require: (klgPackagesDirectory // 'MoreMouseActions.pck.st') pathName	! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 5/25/2022 21:54:20'!
installHaverPackagesFromUnzippedDistribution: aRootDirectory
	"Install all the haver packages from an unzipped distribution."

	| packagesDirectory environmentsDirectory coreChangesDirectory klgPackagesDirectory haverDirectory |
	packagesDirectory _ aRootDirectory / 'Packages' / 'Haver'.
	environmentsDirectory _ packagesDirectory / 'environments'.
	coreChangesDirectory _ aRootDirectory / 'CoreChanges'.
	haverDirectory _ packagesDirectory / 'haver'.
	"Note: We assume that we delivered packages , that passed the checks,
	therefore we do not file them in here,".
	klgPackagesDirectory _ packagesDirectory / 'klg-packages'.
	self killAllBrowsers.
	(coreChangesDirectory // 'CuisCoreEnvironmentSupport.cs.st') readStream fileIn.
	Feature
		require: (klgPackagesDirectory // 'Accessability.pck.st') pathName;
		require: (klgPackagesDirectory // 'KeyboardFocus.pck.st') pathName;
		require: (environmentsDirectory // 'SimpleEnvironments.pck.st') pathName;
		require: (environmentsDirectory // 'Environments.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'Modules.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'ModulesRefactorings.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'HaverCompiler.pck.st') pathName;
		require: (haverDirectory / 'ui' // 'DNDIndentingListItemMorph.pck.st') pathName;
		require: (haverDirectory / 'actions' // 'ActionBuilder.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'ModulesTools.pck.st') pathName;
		"F:
		require: (haverDirectory / 'transactions' // 'WriteBarrier.pck.st') pathName;
		require: (haverDirectory / 'transactions' // 'Transactions.pck.st') pathName"
		require: (haverDirectory / 'modules' // 'ClassBuilding.pck.st') pathName;
		require: (haverDirectory / 'modules' // 'ModuleAwareTools.pck.st') pathName;
		require: (klgPackagesDirectory // 'PreferencesMenu.pck.st') pathName;
		require: (klgPackagesDirectory // 'MoreMouseButtons.pck.st') pathName	;
		require: (klgPackagesDirectory // 'MoreMouseActions.pck.st') pathName! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 9/23/2021 21:09:29'!
isUnzippedDistribution
	"Answer true if we run in an unzipped distribution."

	^ isUnzippedDistribution ifNil: [
		isUnzippedDistribution _ DirectoryEntry smalltalkImageDirectory parent //
			self unzippedDistributionIndicatorFilename ::
				exists ]! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 9/22/2021 20:20:55'!
killAllBrowsers
	"Kill all the browser windows."

	^ (WorldMorph runningWorld
		submorphsSatisfying: [ :morph |
			(morph is: #SystemWindow) and: [ morph model isKindOf: Browser ]]) 
		do: [ :browserWindow |
			browserWindow delete ]! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 4/22/2022 10:05:40'!
postPackageInstall
	"This gets called after installing all the package code.
	Redefine as appropriate"
	
	Transcript cr; show: 'Installing Haver ...'.
	self shouldIntegrate ifTrue: [
		StdIOWriteStream stdout
			nextPutAll: 'About to create a Haver image.';
			newLine;
			nextPutAll: 'Please standby ...';
			flush ].
	self installHaver.
	Transcript cr; show: '... Haver installed'.
	self runningWorld ifNotNil: [ :world |
		world backgroundImageData: nil ].
	HaverDarkPhisherBryceTheme beCurrent.
	self ensureHaverPreferences.
	self shouldIntegrate ifTrue: [
		StdIOWriteStream stdout			
			nextPutAll: ' image created!!';
			newLine;
			flush.
		Smalltalk saveAsNewHaverVersion ]! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 9/13/2021 15:24:38'!
prePackageInstall
	"This gets called after creating the package class and installing its code, but before installing the rest of the package code
	Redefine as appropriate"
	
	Utilities setAuthorName: 'Gerald Klix (Haver)' initials: 'KLG'.
	ChangeSet installNewUpdates! !

!HaverizePackage class methodsFor: 'installing' stamp: 'KLG 5/29/2022 15:02:51'!
shouldIntegrate
	"Answer true if we should integrate Haver."

	^ self hasIntegrateCommandlineOption! !
HaverizePackage prePackageInstall!
!classDefinition: #HaverDarkTheme category: 'Haverize'!
DarkTheme subclass: #HaverDarkTheme
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Haverize'!
!classDefinition: 'HaverDarkTheme class' category: 'Haverize'!
HaverDarkTheme class
	instanceVariableNames: ''!

!classDefinition: #HaverDarkPhisherBryceTheme category: 'Haverize'!
HaverDarkTheme subclass: #HaverDarkPhisherBryceTheme
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Haverize'!
!classDefinition: 'HaverDarkPhisherBryceTheme class' category: 'Haverize'!
HaverDarkPhisherBryceTheme class
	instanceVariableNames: ''!

!classDefinition: #HaverDarkRectangularPhisherBryceTheme category: 'Haverize'!
HaverDarkPhisherBryceTheme subclass: #HaverDarkRectangularPhisherBryceTheme
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Haverize'!
!classDefinition: 'HaverDarkRectangularPhisherBryceTheme class' category: 'Haverize'!
HaverDarkRectangularPhisherBryceTheme class
	instanceVariableNames: ''!


!HaverDarkTheme commentStamp: '<historical>' prior: 0!
Haver's custom dark theme.!

!HaverDarkPhisherBryceTheme commentStamp: '<historical>' prior: 0!
Haver's custom theme, with syntax-highlighting inspired by the product design of a big toy manufacturer.!

!HaverDarkTheme methodsFor: 'colors' stamp: 'KLG 12/22/2020 15:27:43'!
background
	^ `Color darkGray darker darker`! !

!HaverDarkTheme methodsFor: 'colors' stamp: 'KLG 12/22/2020 15:27:43'!
errorColor
	^ `Color red darker`! !

!HaverDarkTheme methodsFor: 'colors' stamp: 'KLG 12/22/2020 15:27:43'!
failureColor
	^ `Color orange darker`! !

!HaverDarkTheme methodsFor: 'colors' stamp: 'KLG 12/22/2020 15:27:43'!
successColor
	^ `Color green darker`! !

!HaverDarkTheme methodsFor: 'colors' stamp: 'KLG 12/22/2020 15:27:43'!
unfocusedTextHighlightFrom: aColor
	^ aColor adjustSaturation: -0.25 brightness: -0.1! !

!HaverDarkTheme methodsFor: 'menu colors' stamp: 'KLG 12/22/2020 15:27:43'!
menu
	^ `Color darkGray lighter`! !

!HaverDarkTheme methodsFor: 'menu colors' stamp: 'KLG 12/22/2020 15:27:43'!
menuText
	^ `Color black`! !

!HaverDarkTheme methodsFor: 'tool colors' stamp: 'KLG 5/9/2021 11:11:40'!
autoCompleterMaybeInvalid
	"Answer the autocompleter's color for possible invalid entries."

	^ `Color lightOrange`! !

!HaverDarkTheme methodsFor: 'tool colors' stamp: 'KLG 12/22/2020 15:27:43'!
debugger
	^ `Color brown`! !

!HaverDarkTheme methodsFor: 'tool colors' stamp: 'KLG 5/9/2021 10:50:09'!
fileContentsBrowser
	^ `Color tan muchDarker`! !

!HaverDarkTheme methodsFor: 'tool colors' stamp: 'KLG 12/22/2020 15:27:43'!
object
	^ `Color gray`! !

!HaverDarkPhisherBryceTheme methodsFor: 'private - shout mappings' stamp: 'KLG 7/3/2021 20:13:28'!
generateShoutConfig

	| answer |
	answer _ super generateShoutConfig.
	answer
		add: `{#string. Color lightGreen. #normal}`;
		add: `{#classVar. Color yellow darker. #bold}`;
		add: `{#incompleteIdentifier . Color orange. {#italic. #underlined}}`.
	^ answer ! !

!HaverDarkPhisherBryceTheme methodsFor: 'colors' stamp: 'KLG 7/3/2021 20:21:35'!
shout
	"Color symbols as an association list."

	^ {
		#selector  					-> '#B59F60'.
		#arguments 					-> Color pink.
		#comment 					-> #(green duller).
		#tempBar 					-> Color red muchLighter.
		#tempVars 					->  '#00F024'.
		#instVar 					-> Color yellow darker.
		#pseudoVariables 			-> Color cyan darker.
		#literals 					-> #green.
		#messages 					-> '#6FB3BD'.
		#incompleteMessages  	-> '#F08060'.
		#blockLevelZero 			-> '#6FB3BD'.
		#blockLevelOne 			-> '#FFB0B0'.
		#blockLevelTwo 			-> '#B0FFB0'.
		#blockLevelThree 			-> '#B0B0FF'.
		#blockLevelFour 			-> '#00B0B0'.
		#blockLevelFive 			-> '#E03030'.
		#blockLevelSix 			-> '#30E030'.
		#blockLevelSeven 			-> '#3030E0'.
		#defaults 					-> '#A1AFBF'.
		#undefined 					-> '#E04020'.
		#methodTags 				-> #green.
	}! !

!HaverDarkPhisherBryceTheme methodsFor: 'tool colors' stamp: 'KLG 5/9/2021 11:10:37'!
autoCompleter
	"Answer the autocompleter's color for  entries."

	^ `Color lightGray lighter`! !

!HaverDarkPhisherBryceTheme methodsFor: 'tool colors' stamp: 'KLG 5/9/2021 11:08:06'!
autoCompleterDefaultBorderColor
	"Answer the auto comleters default border color."

	^ `Color black`! !

!HaverDarkPhisherBryceTheme methodsFor: 'widget colors' stamp: 'KLG 12/9/2021 19:01:24'!
hoverHelp
	"Answer the hover help morph's background color."

	^ `Color lightBlue`! !

!HaverDarkPhisherBryceTheme methodsFor: 'widget colors' stamp: 'KLG 12/9/2021 19:04:21'!
hoverHelpBorderColor
	"Answer the color for Hover help widgets."

	^ `Color orange darker`! !

!HaverDarkPhisherBryceTheme methodsFor: 'other options' stamp: 'KLG 5/17/2022 21:42:40'!
hoverHelpBorderWidth
	"Answer the hover help morph's border width."
	
	^ 6! !

!HaverDarkRectangularPhisherBryceTheme methodsFor: 'other options' stamp: 'KLG 11/29/2021 12:21:15'!
roundWindowCorners
	"I have retangular corners."

	^ false! !

!ProtoObject methodsFor: '*Haverize-debugging' stamp: 'KLG 1/3/2021 16:56:40'!
flagHaver: aComplaintStringOrSymbol
	"Flag something concernig haver."! !

!SystemDictionary methodsFor: '*Haverize-image, changes name' stamp: 'KLG 4/11/2021 17:56:59'!
baseNameForVersion: aSystemVersion distributionPrefix: aPrefixString
	"Answer a new haver base name for a system version."

	^ String streamContents: [ :strm |
		strm
			nextPutAll: aPrefixString;
			print: aSystemVersion versionMajor;
			nextPut: $.;
			print: aSystemVersion versionMinor;
			nextPut: $-;
			print: aSystemVersion highestUpdate.
		Smalltalk isSpur
			ifTrue: [	
				Smalltalk wordSize = 4 ifTrue: [
					strm nextPutAll: '-32' ]]
			ifFalse: [
				strm nextPutAll: '-v3' ]]! !

!SystemDictionary methodsFor: '*Haverize-image, changes name' stamp: 'KLG 4/11/2021 17:58:31'!
cuisBaseNameForCurrentVersion
	"Answer a new cuis base name for the running system."

	^ self cuisBaseNameForVersion: SystemVersion current! !

!SystemDictionary methodsFor: '*Haverize-image, changes name' stamp: 'KLG 4/11/2021 17:58:12'!
cuisBaseNameForVersion: aSystemVersion
	"Answer a new cuis base name for a system version."

	^ self baseNameForVersion: aSystemVersion distributionPrefix: 'Cuis'! !

!SystemDictionary methodsFor: '*Haverize-image, changes name' stamp: 'KLG 4/11/2021 17:59:03'!
haverBaseNameForCurrentVersion
	"Answer a new haver base name for the running system."

	^ self haverBaseNameForVersion: SystemVersion current! !

!SystemDictionary methodsFor: '*Haverize-image, changes name' stamp: 'KLG 4/11/2021 17:56:59'!
haverBaseNameForVersion: aSystemVersion
	"Answer a new haver base name for a system version."

	^ self baseNameForVersion: aSystemVersion distributionPrefix: 'Haver'! !

!SystemDictionary methodsFor: '*Haverize-snapshot and quit' stamp: 'KLG 4/23/2021 12:10:23'!
saveAsNewHaverVersion
	"Save the image/changes using the next available version number."
	"
	Smalltalk saveAsNewHaverVersion
	"
	self saveAsNewHaverVersion: true! !

!SystemDictionary methodsFor: '*Haverize-snapshot and quit' stamp: 'KLG 9/7/2021 17:13:59'!
saveAsNewHaverVersion: aQuitFlag
	"Save the image/changes using the next available version number."

	| fileName newName changesName |
	fileName _ self haverBaseNameForCurrentVersion.
	newName _ fileName, '.image'.
	(DirectoryEntry smalltalkImageDirectory // newName) exists ifTrue: [
		newName _ DirectoryEntry smalltalkImageDirectory
			nextNameFor: fileName
			extension: 'image' ].
	changesName _ self fullNameForChangesNamed: newName.
	"Check to see if there is a .changes file that would cause a problem if we saved a new .image file with the new version number"
	changesName asFileEntry exists ifTrue: [
		^ self inform:
'There is already .changes file of the desired name,
', changesName, '
curiously already present, even though there is no
corresponding .image file.
Please remedy manually and then repeat your request.' ].
	"Try to clear all user state, including all class vars, preferences, etc"
	"We do want a clean image!!"
	aQuitFlag ifTrue: [ ChangeSet nukeAllChangeSetsForHaverDelivery ].
	self
		saveAndQuitAs: newName
		clearAllClassState: true
! !

!SystemDictionary class methodsFor: '*Haverize-menu-world' stamp: 'KLG 4/28/2021 14:14:46'!
worldMenuOptions

	^`{{
		#itemGroup 				-> 		30.
		#itemOrder 				-> 		32.
		#label 				-> 		'Save new Haver Release'.
		#object 				-> 		Smalltalk.
		#selector 				-> 		#saveAsNewHaverVersion.
		#icon 				-> 		#saveAsNewVersionIcon.
		#balloonText 				-> 		('Save as a new release of Haver.
Use an updated version-stamped name
and save it under that name on disk.

', 'Clear' bold, ' all user preferences and user state (class vars).
Then ', 'quit' bold, ' running the image.

','This action is for the maintainer of Haver!!' red bold, '
Use at your own risk.' red).
	} asDictionary }`! !

!Transcripter methodsFor: '*Haverize-command line' stamp: 'KLG 12/22/2020 15:23:08'!
readEvalPrint
	| line |
	[ #('quit' 'exit' 'done' ) includes: (line _ self request: '>') ] whileFalse: [
		line caseOf: {
			[ 'revert' ] -> [ self newLine; show: '"revert" not implemented (yet)"'].
			[ 'bailout' ] -> [ Smalltalk quitPrimitive: 42 ]}
		otherwise: [
			self
				newLine;
				show:
					([ Compiler evaluate: line ] ifError: [ :err :ex | err ]) ]]! !

!SystemVersion methodsFor: '*Haverize-accessing' stamp: 'KLG 12/23/2020 18:49:17'!
version
	"Answer the haver version"

	^ String streamContents: [ :strm |
		strm nextPutAll: 'Haver '.
		versionMajor printOn: strm.
		strm nextPut: $..
		versionMinor printOn: strm ].! !

!Utilities class methodsFor: '*Haverize-common requests' stamp: 'KLG 12/23/2020 18:55:52'!
saveDisplay
	"Save Display to a file"

	"SampledSound playSoundNamed: 'camera'."
	Display writeBMPfileNamed: 'HaverDisplay.bmp'! !

!Utilities class methodsFor: '*Haverize-common requests' stamp: 'KLG 12/23/2020 18:56:05'!
saveScreenshot
	"Make a screenshot of the world and save it to a file"

	"SampledSound playSoundNamed: 'camera'."
	(self runningWorld imageForm: 32) writeBMPfileNamed: 'HaverWorld.bmp'! !

!WorldMorph methodsFor: '*Haverize-default desktop' stamp: 'KLG 5/17/2022 21:31:19'!
recreateDefaultDesktop
	
	self whenUIinSafeState: [ | menu |
		self hideTaskbar.
		(submorphs
			select: [ :ea | ea class == SystemWindow or: [ ea class == TranscriptWindow ]])
				do: [ :ea | ea delete ].
		Theme current useTaskbar ifTrue: [ self showTaskbar ].
		menu _ TheWorldMenu new 
			world: self
			hand: self activeHand :: buildWorldMenu.
		menu
			addTitle: PreferenceNG desktopMenuTitle;
			popUpInWorld: self;
			stayUp 	]! !

!IndentingListItemMorph methodsFor: '*Haverize-initialization' stamp: 'KLG 10/27/2020 11:46:06'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Theme current text! !

!SmalltalkCompleterMorph methodsFor: '*Haverize-selector documentation' stamp: 'KLG 5/10/2021 20:25:35'!
initializeSelectorDocumentation

	selectorDocumentation := TextModelMorph withText: ''.
	selectorDocumentation textMorph
		acceptOnCR: false;
		crAction: [ self crPressedOnSelectorDocumentation ].

	selectorDocumentation
		wrapFlag: true;
		borderColor: Color black;
		color: self color;
		borderWidth: 1;
		disableEditing;
		openInWorld
	! !

!TextComposition methodsFor: '*Haverize-display' stamp: 'KLG 12/1/2020 09:23:39'!
displayTextCursorAtX: x top: top bottom: bottom emphasis: emphasis on: aCanvas textLeft: textLeft
	"x, top, bottom, textLeft are relative to the morph currently being drawn."

	| textCursorColor x1 isBold isItalic x0 h w halfW r d extraW corner origin
	  finalStart finalEnd 
	  flangeHeight flangeWidth |
	isBold _ emphasis allMask: 1.
	isItalic _ emphasis allMask: 2.
	textCursorColor _ Theme current textCursor.
	h _ bottom - top.
	w _ isBold
		ifTrue: [ h // 25 + 2 ]
		ifFalse: [ h // 30 + 1 ].
	halfW _ w // 2.
	isItalic
		ifTrue: [	
			"Keep tweaking if needed!! For italics with descenders (i.e. p), cursor shows a bit to the left..."
			d _ isBold ifTrue: [ h // 8 ] ifFalse: [ h // 9].
			x0 _ x- (h*5//24) + d.
			x1 _ x + d ]
		ifFalse: [
			x0 _ x - halfW.
			x1 _ x - halfW].
	x0-textLeft < -1 ifTrue: [
		x1 _ x1 - x0 + textLeft.
		x0 _ textLeft ].
	r _ extentForComposing x-halfW-1.
	r < (x1-textLeft) ifTrue: [
		x0 _ x0 + r - x1+textLeft.
		x1 _ r +textLeft].
	extraW _ isBold
		ifTrue: [ 3 ]
		ifFalse: [ 2 ].
	origin _ x0-halfW@ top.
	corner _ x1+halfW+extraW @ (bottom+2).
	flangeHeight _ 1.
	flangeWidth _ 5.
	lastTextCursorRect
		ifNil: [ lastTextCursorRect _ origin corner: corner ]
		ifNotNil: [lastTextCursorRect
			setOrigin: (lastTextCursorRect origin min: origin)
			corner: (lastTextCursorRect corner max: corner)].
	lastTextCursorRect _
		lastTextCursorRect expandBy: (flangeWidth @ flangeHeight).
	aCanvas
		line: (finalStart _ (x0+halfW@(bottom-w-w+halfW) - (0 @ flangeHeight)))
		 	to: (finalEnd _ (x1+halfW@top) + (0 @ flangeHeight))
			width: w color: textCursorColor;
		line: finalEnd - (flangeWidth @ flangeHeight) to: finalEnd
			width:  w color: textCursorColor;
		line: finalEnd to: finalEnd + (flangeWidth @ flangeHeight negated)
			width:  w color: textCursorColor;
		line: finalStart + (flangeWidth @ flangeHeight) to: finalStart
			width:  w  color: textCursorColor;
		line: finalStart to: finalStart - (flangeWidth @ flangeHeight negated)
			width:  w  color: textCursorColor
! !

!ChangeSet class methodsFor: '*Haverize-removing' stamp: 'KLG 4/24/2021 14:12:55'!
nukeAllChangeSetsForHaverDelivery
	"Realy nuke all change sets, because we wnat to deliver the image
	and we filed in some base system changes."
	
	AllChangeSets copy do: [ :changeSet | 		self removeChangeSet: changeSet ]! !
HaverizePackage postPackageInstall!
