'From Haver 6.0 [latest update: #5173] on 18 May 2022 at 8:22:04 pm'!
'Description I provide modifications that support zooming Toplevel windows.'!
!provides: 'MorphZooming' 1 15!
!requires: 'VectorGraphics' 1 266 nil!






!Morph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/17/2022 22:23:50'!
handlesMouseScroll: aMouseScrollEvent
	"Answer true if we handle mouse scroll events."
	
	^ self wantsMorphZooming and: [ aMouseScrollEvent rawMacOptionKeyPressed ] :: or: [
		self hasProperty: #'handlesMouseScroll:' ]! !

!Morph methodsFor: '*MorphZooming-events' stamp: 'KLG 5/18/2022 19:19:24'!
mouseScroll: aMouseScrollEvent localPosition: localEventPosition
	"Handle a mouse scroll event.
	This message will only be sent to Morphs that answer true to #handlesMouseScroll:
	We can query aMouseScrollEvent to know about pressed mouse buttons."
	
	"Allow instances to dynamically use properties for handling common events."

	| scaleFactor newScale |
	aMouseScrollEvent rawMacOptionKeyPressed 
		ifTrue: [
			scaleFactor _ aMouseScrollEvent direction
				caseOf: {
					[ #up ] -> [ 1.05 ].
					[ #down ] -> [ `1 / 1.05` ].
					[ #left ] -> [ `1 / 1.5` ].
					[ #right ] -> [ 1.5 ] } ].
	scaleFactor ifNotNil: [
		newScale _ self scale * scaleFactor.
		newScale > 5 ifTrue: [ self flash ].
		newScale < 0.5 ifTrue: [ self flash ].
		newScale _ newScale min: 5 max: 0.5.
		(newScale - 1) abs < 0.01
			ifTrue: [ newScale _ 1 ].
		^ self zoomMorphBy: newScale ].
	self 
		valueOfProperty: #mouseScroll:localPosition: 
		ifPresentDo: [ :handler | handler value: aMouseScrollEvent value: localEventPosition ]! !

!Morph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/17/2022 22:27:11'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ true! !

!Morph methodsFor: '*MorphZooming-geometry' stamp: 'KLG 5/18/2022 19:18:46'!
zoomMorphBy: aScale
	"Zoom the morph by a scale.
	
	Defaults to scale, but can be overridden in subclasses."

	self scale: aScale! !

!BoxedMorph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/18/2022 19:06:36'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ owner 
		ifNil: [ super wantsMorphZooming ]
		ifNotNil: [ owner is: #LayoutMorph ::
			ifFalse: [ owner isKindOf: MenuMorph :: not]
			ifTrue:  [ owner owner
				ifNotNil: [ :oo | oo isKindOf: MenuMorph :: not ]
				ifNil: [ true ] ] ]! !

!WorldMorph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/18/2022 11:36:46'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ false! !

!LayoutMorph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/17/2022 22:33:55'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ false! !

!InnerListMorph methodsFor: '*MorphZooming-geometry' stamp: 'KLG 5/18/2022 19:38:03'!
zoomMorphBy: aScale
	"Zoom the morph by a scale.
	
	Defaults to scale, but can be overridden in subclasses."
	
	owner ifNil: [ ^ super zoomMorphBy: aScale ].
	owner zoomMorphBy: aScale! !

!LabelMorph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/18/2022 19:02:24'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ owner 
		ifNil: [ super wantsMorphZooming ]
		ifNotNil: [ 
			owner is: #BoxedMorph ::
				ifTrue:  [ owner wantsMorphZooming ] 
				ifFalse: [ super wantsMorphZooming ] ]! !

!IndentingListItemMorph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/18/2022 20:07:22'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ false! !

!MenuItemMorph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/18/2022 11:37:03'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ false! !

!PluggableButtonMorph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/17/2022 22:36:11'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ false! !

!PluggableScrollPane methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/18/2022 20:05:27'!
handlesMouseScroll: aMouseScrollEvent
	"Only accept if we can actually do something useful with the event (i.e. not scrolling up when already at the top or down when already at the bottom) or if my owner chain doesn't want it"

	(aMouseScrollEvent direction = #up and: [ scrollBar scrollValue > 0 ])
		ifTrue: [ ^ true ].
	(aMouseScrollEvent direction = #down and: [ scrollBar scrollValue < 1 ])
		ifTrue: [ ^ true ].
	(aMouseScrollEvent direction = #left and: [ hScrollBar scrollValue > 0 ])
		ifTrue: [ ^ true ].
	(aMouseScrollEvent direction = #right and: [ hScrollBar scrollValue < 1 ])
		ifTrue: [ ^ true ].
	"Even if I don't want it, one of my owners might. (i.e. nested scroll panes)  If my owners don't want it, accept the event to make sure that morphs behind me doesn't get the event."
	(self privateAnyOwnerHandlesMouseScroll: aMouseScrollEvent)
		ifFalse: [ ^ true ].
	^ super handlesMouseScroll: aMouseScrollEvent! !

!PluggableScrollPane methodsFor: '*MorphZooming-events' stamp: 'KLG 5/18/2022 20:04:11'!
mouseScroll: aMouseEvent localPosition: localEventPosition
	"Scroll the the pane."
	
	"Take care of zoom events:"
	aMouseEvent rawMacOptionKeyPressed ifTrue: [
		^ super mouseScroll: aMouseEvent localPosition: localEventPosition ].
	aMouseEvent direction
		caseOf: {
			[ #up ] 		-> 		[  scrollBar scrollUp: 1 ].
			[ #down ] 	-> 		[ scrollBar scrollDown: 1 ].
			[ #left ] 	-> 		[  hScrollBar scrollUp: 1 ].
			[ #right ] 	-> 		[  hScrollBar scrollDown: 1 ] }! !

!PluggableListMorph methodsFor: '*MorphZooming-geometry' stamp: 'KLG 5/18/2022 19:47:52'!
zoomMorphBy: aScale
	"Zoom the morph by a scale.
	
	Defaults to scale, but can be overridden in subclasses."

	self submorphsDo: [ :subMorph | subMorph scale: aScale ].
	scrollSiblings ifTrue: [
		leftSibling ifNotNil: [ leftSibling zoomMorphToTheLeftBy: aScale ].
		rightSibling ifNotNil: [ rightSibling zoomMorphToTheRightBy: aScale ] ]! !

!PluggableListMorph methodsFor: '*MorphZooming-geometry' stamp: 'KLG 5/18/2022 19:46:44'!
zoomMorphToTheLeftBy: aScale
	"Zoom the morph by a scale. Propagte to the left sibling chain."

	self submorphsDo: [ :subMorph | subMorph scale: aScale ].
	scrollSiblings ifTrue: [
		leftSibling ifNotNil: [ leftSibling zoomMorphToTheLeftBy: aScale ] ]! !

!PluggableListMorph methodsFor: '*MorphZooming-geometry' stamp: 'KLG 5/18/2022 19:46:25'!
zoomMorphToTheRightBy: aScale
	"Zoom the morph by a scale. Propagte to the right sibling chain."

	self submorphsDo: [ :subMorph | subMorph scale: aScale ].
	scrollSiblings ifTrue: [
		rightSibling ifNotNil: [ rightSibling zoomMorphToTheRightBy: aScale ] ]! !

!SystemWindow methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/16/2022 19:32:37'!
handlesMouseScroll: aMouseScrollEvent
	"Answer true if we want mouse scroll events."
	
	^ true! !

!SystemWindow methodsFor: '*MorphZooming-events' stamp: 'KLG 5/16/2022 20:48:41'!
mouseScroll: aMouseScrollEvent localPosition: localEventPosition
	"Handle a mouse scroll event."

	| scaleFactor newScale |
	aMouseScrollEvent controlKeyPressed
		ifTrue: [
			scaleFactor _ aMouseScrollEvent direction
				caseOf: {
					[ #up ] -> [ 1.05 ].
					[ #down ] -> [ `1 / 1.05` ].
					[ #left ] -> [ `1 / 1.2` ].
					[ #right ] -> [ 1.2 ] } ].
	scaleFactor ifNil: [ ^ self ].
	newScale _ self scale * scaleFactor min: 5 max: 0.5.
	(newScale - 1) abs < 0.01
		ifTrue: [ newScale _ 1 ].
	self scale: newScale! !

!(Modules>>#LegacyButtons>>#PluggableButtonMorph) methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/17/2022 22:39:08'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ false! !

!MenuLineMorph methodsFor: '*MorphZooming-event handling testing' stamp: 'KLG 5/18/2022 11:37:13'!
wantsMorphZooming
	"Answer true if this morph want's morph zooming using hot keys or scroll events."
	
	^ false! !

!KeyboardEvent methodsFor: '*MorphZooming-testing' stamp: 'KLG 5/16/2022 20:24:47'!
isFastMorphUnzoom

	^ (self controlKeyPressed or: [ self rawMacOptionKeyPressed ]) and: [
		self keyCharacter = $< ]! !

!KeyboardEvent methodsFor: '*MorphZooming-testing' stamp: 'KLG 5/16/2022 20:24:54'!
isFastMorphZoom

	^ (self controlKeyPressed or: [ self rawMacOptionKeyPressed ]) and: [ 
		self keyCharacter = $> ]! !

!KeyboardEvent methodsFor: '*MorphZooming-testing' stamp: 'KLG 5/17/2022 22:10:30'!
isScaleMinusWindowShortcut

	^ self controlKeyPressed and: [ self keyCharacter = $- ]! !

!KeyboardEvent methodsFor: '*MorphZooming-testing' stamp: 'KLG 5/17/2022 22:11:33'!
isScaleOneWindowShortcut

	^ self controlKeyPressed and: [ self keyCharacter = $0 ]! !

!KeyboardEvent methodsFor: '*MorphZooming-testing' stamp: 'KLG 5/17/2022 22:12:39'!
isScalePlusWindowShortcut

	^ self controlKeyPressed and: [ self keyCharacter = $+ ]! !

!KeyboardEvent methodsFor: '*MorphZooming-testing' stamp: 'KLG 5/18/2022 20:17:30'!
isZoomMinusMorphShortcut

	^ self rawMacOptionKeyPressed and: [ self keyCharacter = $- ]! !

!KeyboardEvent methodsFor: '*MorphZooming-testing' stamp: 'KLG 5/18/2022 20:17:49'!
isZoomOneMorphShortcut

	^ self rawMacOptionKeyPressed and: [ self keyCharacter = $0 ]! !

!KeyboardEvent methodsFor: '*MorphZooming-testing' stamp: 'KLG 5/18/2022 20:18:10'!
isZoomPlusMorphShortcut

	^ self rawMacOptionKeyPressed and: [ self keyCharacter = $+ ]! !

!KeyboardEvent methodsFor: '*MorphZooming-actions' stamp: 'KLG 11/29/2021 19:47:49'!
scaleOneCurrentWindowOf: aMorph

	aMorph owningWindow ifNotNil: [ :w |
		w scale: 1 ]! !

!KeyboardEvent methodsFor: '*MorphZooming-actions' stamp: 'KLG 5/16/2022 19:16:53'!
scalePlusCurrentWindowOf: aMorph 
	"The + key was used to scale the window"
	
	aMorph owningWindow ifNotNil: [ :w |
		w scale: w scale * 1.05 ]! !

!KeyboardEvent methodsFor: '*MorphZooming-dispatching' stamp: 'KLG 5/18/2022 20:18:10'!
sendEventTo: aMorph
	"Dispatch the receiver into anObject"

	type == #keystroke ifTrue: [
		self isFindClassShortcut
			ifTrue: [ ^ PreferenceNG at: #classFinder :: value ].
		self isCloseWindowShortcut
			ifTrue: [ ^ self closeCurrentWindowOf: aMorph ].
		self isScaleOneWindowShortcut ifTrue: [
			^ self scaleOneCurrentWindowOf: aMorph ].
		self isScaleMinusWindowShortcut ifTrue: [
			^ self zoomMinusCurrentWindowOf: aMorph ].
		self isScalePlusWindowShortcut ifTrue: [
			^ self scalePlusCurrentWindowOf: aMorph ].
		aMorph wantsMorphZooming ifTrue: [
			self isZoomOneMorphShortcut ifTrue: [
				^ self zoomOneForMorph: aMorph ].
			self isZoomMinusMorphShortcut ifTrue: [
				^ self zoomMinusForMorph: aMorph ].
			self isZoomPlusMorphShortcut ifTrue: [
				^ self zoomPlusForMorph: aMorph ].
			self isFastMorphZoom ifTrue: [
				^ self zoomForMorph: aMorph ].
			self isFastMorphUnzoom ifTrue: [
				^ self unzoomForMorph: aMorph ] ].
		^ aMorph processKeystroke: self ].
	type == #keyDown ifTrue: [
		^ aMorph processKeyDown: self ].
	type == #keyUp ifTrue: [
		^ aMorph processKeyUp: self ].
	^ super sendEventTo: aMorph.! !

!KeyboardEvent methodsFor: '*MorphZooming-actions' stamp: 'KLG 5/18/2022 19:20:22'!
unzoomForMorph: aMorph 
	"unZoom the current morph."
	
	aMorph zoomMorphBy: (aMorph valueOfProperty: #unzoomedScale ifAbsent: [ 1 ]).
	aMorph removeProperty: #unzoomedScale! !

!KeyboardEvent methodsFor: '*MorphZooming-actions' stamp: 'KLG 5/18/2022 19:20:31'!
zoomForMorph: aMorph 
	"Zoom the current morph."
	
	aMorph hasProperty: #unzoomedScale :: ifTrue: [ ^ self ].
	(aMorph setProperty: #unzoomedScale toValue: aMorph scale) < 2.5 ifTrue: [
		aMorph zoomMorphBy: 2.5 ]! !

!KeyboardEvent methodsFor: '*MorphZooming-actions' stamp: 'KLG 5/18/2022 20:14:36'!
zoomMinusCurrentWindowOf: aMorph 
	"The - key was used to scale the window"
	
	aMorph owningWindow ifNotNil: [ :w |
		w scale: w scale / 1.05 ]! !

!KeyboardEvent methodsFor: '*MorphZooming-actions' stamp: 'KLG 5/18/2022 20:15:15'!
zoomMinusForMorph: aMorph 
	"The - key was used to scale the morph"
	
	aMorph zoomMorphBy: aMorph scale / 1.05! !

!KeyboardEvent methodsFor: '*MorphZooming-actions' stamp: 'KLG 5/18/2022 20:15:37'!
zoomOneForMorph: aMorph
	"Zoom aMorph back to szize one."

	aMorph
		zoomMorphBy: 1;
		removeProperty: #unzoomedScale
	! !

!KeyboardEvent methodsFor: '*MorphZooming-actions' stamp: 'KLG 5/18/2022 20:16:00'!
zoomPlusForMorph: aMorph 
	"The - key was used to scale the window"
	
	aMorph zoomMorphBy: aMorph scale * 1.05! !
